# PrinCompStability
#
# Function 'PrinCompStability' computes the stability of the
# principal components computed from matrices S or R, according to Krzanowski (1984b).
#
# Inputs: X data matrix
#         alpha perturbation parameter (alpha=0.1, 0.05, 0.01)
#
# Outputs: DpS perturbed eigenvalues of S,
#          TpS perturbed eigenvectors of S,
#          thetaS angle (in degrees) between the perturbed eigenvectors and the original ones computed from S,
#          DpR perturbed eigenvalues of R,
#          TpR perturbed eigenvectors of R,
#          thetaR angle (in degrees) between the perturbed eigenvectors and the original ones computed from R.
#
PrinCompStability<-function(X,alpha=0.05)
{
  dims<-dim(X)
  S<-cov(X)
  R<-cor(X)
  outS<-eigen(S)
  outR<-eigen(R)
  T1<-outS$vectors
  T2<-outR$vectors
  D1<-outS$values
  D2<-outR$values
  #auxilary routine
  find.perturbed<-function(T0,D0,alpha=alpha)
  {
    epsilon<-alpha*D0
    p<-length(D0)
    Tp<-theta<-Dp<-NULL
    for(j in 1:(p-1))
    {
      Tp<-cbind(Tp, (T0[,j]+T0[,j+1]*sqrt(epsilon[j]/(D0[j]-D0[j+1])))/sqrt(1+epsilon[j] / (D0[j]-D0[j+1])) )
      theta<-c(theta,acos(1/sqrt(1+epsilon[j]/(D0[j]-D0[j+1]))) )
      Dp<-c(Dp, D0[j]-epsilon[j]) 
    } 
    theta<-theta*360/(2*pi)
    list(Tp=Tp,Dp=Dp,theta=theta)
  }  
  outS<-find.perturbed(T1,D1,alpha=alpha)
  outR<-find.perturbed(T2,D2,alpha=alpha)
  list(DpS=outS$Dp,TpS=outS$Tp,thetaS=outS$theta,
       DpR=outR$Dp,TpR=outR$Tp,thetaR=outR$theta)
}  
